#
# Copyright 2005,2006,2011 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#

"""
PSK modulation and demodulation.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from math import pi, log
from cmath import exp

from . import digital_swig
from . import modulation_utils
from .utils import mod_codes, gray_code
from .generic_mod_demod import generic_mod, generic_demod
from .generic_mod_demod import shared_mod_args, shared_demod_args

# Default number of points in constellation.
_def_constellation_points = 4
# The default encoding (e.g. gray-code, set-partition)
_def_mod_code = mod_codes.GRAY_CODE
# Default use of differential encoding
_def_differential = True

def create_encodings(mod_code, arity, differential):
    post_diff_code = None
    if mod_code not in mod_codes.codes:
        raise ValueError('That modulation code does not exist.')
    if mod_code == mod_codes.GRAY_CODE:
        if differential:
            pre_diff_code = gray_code.gray_code(arity)
            post_diff_code = None
        else:
            pre_diff_code = []
            post_diff_code = gray_code.gray_code(arity)
    elif mod_code == mod_codes.NO_CODE:
        pre_diff_code = []
        post_diff_code = None
    else:
        raise ValueError('That modulation code is not implemented for this constellation.')
    return (pre_diff_code, post_diff_code)

# /////////////////////////////////////////////////////////////////////////////
#                           PSK constellation
# /////////////////////////////////////////////////////////////////////////////

def psk_constellation(m=_def_constellation_points, mod_code=_def_mod_code,
                      differential=_def_differential):
    """
    Creates a PSK constellation object.
    """
    k = log(m) / log(2.0)
    if (k != int(k)):
        raise Exception('Number of constellation points must be a power of two.')
    points = [exp(2*pi*(0+1j)*i/m) for i in range(0,m)]
    pre_diff_code, post_diff_code = create_encodings(mod_code, m, differential)
    if post_diff_code is not None:
        inverse_post_diff_code = mod_codes.invert_code(post_diff_code)
        points = [points[x] for x in inverse_post_diff_code]
    constellation = digital_swig.constellation_psk(points, pre_diff_code, m)
    return constellation

# /////////////////////////////////////////////////////////////////////////////
#                           PSK modulator
# /////////////////////////////////////////////////////////////////////////////

class psk_mod(generic_mod):
    """
    Hierarchical block for RRC-filtered PSK modulation.

    The input is a byte stream (unsigned char), treated as a series of packed
    symbols. Symbols are grouped from MSB to LSB.

    The output is the complex modulated signal at baseband, with a given number
    of samples per symbol.

    If "Samples/Symbol" is 2, and "Number of Constellation Points" is 4, a
    single byte contains four symbols, and will produce eight samples.

    Args:
        constellation_points: Number of constellation points (must be a power of two) (integer).
        mod_code: Whether to use a gray_code (digital.mod_codes.GRAY_CODE) or not (digital.mod_codes.NO_CODE).
        differential: Whether to use differential encoding (boolean).
    """
    # See generic_mod for additional arguments
    __doc__ += shared_mod_args

    def __init__(self, constellation_points=_def_constellation_points,
                 mod_code=_def_mod_code,
                 differential=_def_differential,
                 *args, **kwargs):
        constellation = psk_constellation(constellation_points, mod_code, differential)
        super(psk_mod, self).__init__(constellation, differential, *args, **kwargs)

# /////////////////////////////////////////////////////////////////////////////
#                           PSK demodulator
#
# /////////////////////////////////////////////////////////////////////////////

class psk_demod(generic_demod):

    """
    Hierarchical block for RRC-filtered PSK modulation.

    The input is a complex modulated signal at baseband.

    The output is a stream of bytes, each representing a recovered bit.
    The most significant bit is reported first.

    Args:
        constellation_points: Number of constellation points (must be a power of two) (integer).
        mod_code: Whether to use a gray_code (digital.mod_codes.GRAY_CODE) or not (digital.mod_codes.NO_CODE).
        differential: Whether to use differential encoding (boolean).
    """
    # See generic_mod for additional arguments
    __doc__ += shared_mod_args
    def __init__(self, constellation_points=_def_constellation_points,
                 mod_code=_def_mod_code,
                 differential=_def_differential,
                 *args, **kwargs):
        constellation = psk_constellation(constellation_points, mod_code, differential)
        super(psk_demod, self).__init__(constellation, differential, *args, **kwargs)

#
# Add these to the mod/demod registry
#
modulation_utils.add_type_1_mod('psk', psk_mod)
modulation_utils.add_type_1_demod('psk', psk_demod)
modulation_utils.add_type_1_constellation('psk', psk_constellation)



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.1
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _audio_swig
else:
    import _audio_swig

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def high_res_timer_now() -> "gr::high_res_timer_type":
    r"""high_res_timer_now() -> gr::high_res_timer_type"""
    return _audio_swig.high_res_timer_now()

def high_res_timer_now_perfmon() -> "gr::high_res_timer_type":
    r"""high_res_timer_now_perfmon() -> gr::high_res_timer_type"""
    return _audio_swig.high_res_timer_now_perfmon()

def high_res_timer_tps() -> "gr::high_res_timer_type":
    r"""high_res_timer_tps() -> gr::high_res_timer_type"""
    return _audio_swig.high_res_timer_tps()

def high_res_timer_epoch() -> "gr::high_res_timer_type":
    r"""high_res_timer_epoch() -> gr::high_res_timer_type"""
    return _audio_swig.high_res_timer_epoch()
class source(object):
    r"""Proxy of C++ gr::audio::source class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(*args, **kwargs) -> "gr::audio::source::sptr":
        r"""make(int sampling_rate, std::string const device_name="", bool ok_to_block=True) -> source_sptr"""
        return _audio_swig.source_make(*args, **kwargs)
    __swig_destroy__ = _audio_swig.delete_source

# Register source in _audio_swig:
_audio_swig.source_swigregister(source)

def source_make(*args, **kwargs) -> "gr::audio::source::sptr":
    r"""source_make(int sampling_rate, std::string const device_name="", bool ok_to_block=True) -> source_sptr"""
    return _audio_swig.source_make(*args, **kwargs)

class sink(object):
    r"""Proxy of C++ gr::audio::sink class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(*args, **kwargs) -> "gr::audio::sink::sptr":
        r"""make(int sampling_rate, std::string const device_name="", bool ok_to_block=True) -> sink_sptr"""
        return _audio_swig.sink_make(*args, **kwargs)
    __swig_destroy__ = _audio_swig.delete_sink

# Register sink in _audio_swig:
_audio_swig.sink_swigregister(sink)

def sink_make(*args, **kwargs) -> "gr::audio::sink::sptr":
    r"""sink_make(int sampling_rate, std::string const device_name="", bool ok_to_block=True) -> sink_sptr"""
    return _audio_swig.sink_make(*args, **kwargs)

class source_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::audio::source > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(source_sptr self) -> source_sptr
        __init__(source_sptr self, source p) -> source_sptr
        """
        _audio_swig.source_sptr_swiginit(self, _audio_swig.new_source_sptr(*args))

    def __deref__(self) -> "gr::audio::source *":
        r"""__deref__(source_sptr self) -> source"""
        return _audio_swig.source_sptr___deref__(self)
    __swig_destroy__ = _audio_swig.delete_source_sptr

    def make(self, *args, **kwargs) -> "gr::audio::source::sptr":
        r"""make(source_sptr self, int sampling_rate, std::string const device_name="", bool ok_to_block=True) -> source_sptr"""
        return _audio_swig.source_sptr_make(self, *args, **kwargs)

    def history(self) -> "unsigned int":
        r"""history(source_sptr self) -> unsigned int"""
        return _audio_swig.source_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(source_sptr self, int which, int delay)
        declare_sample_delay(source_sptr self, unsigned int delay)
        """
        return _audio_swig.source_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(source_sptr self, int which) -> unsigned int"""
        return _audio_swig.source_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(source_sptr self, int multiple)"""
        return _audio_swig.source_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(source_sptr self) -> int"""
        return _audio_swig.source_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(source_sptr self) -> double"""
        return _audio_swig.source_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(source_sptr self) -> uint64_t"""
        return _audio_swig.source_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(source_sptr self) -> uint64_t"""
        return _audio_swig.source_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(source_sptr self) -> bool"""
        return _audio_swig.source_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(source_sptr self) -> bool"""
        return _audio_swig.source_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(source_sptr self, unsigned int which_input) -> uint64_t"""
        return _audio_swig.source_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(source_sptr self, unsigned int which_output) -> uint64_t"""
        return _audio_swig.source_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(source_sptr self, std::string level)"""
        return _audio_swig.source_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(source_sptr self) -> std::string"""
        return _audio_swig.source_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(source_sptr self) -> int"""
        return _audio_swig.source_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(source_sptr self, int m)"""
        return _audio_swig.source_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(source_sptr self)"""
        return _audio_swig.source_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(source_sptr self) -> bool"""
        return _audio_swig.source_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(source_sptr self, int m)"""
        return _audio_swig.source_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(source_sptr self) -> int"""
        return _audio_swig.source_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(source_sptr self, int i) -> long"""
        return _audio_swig.source_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(source_sptr self, long max_output_buffer)
        set_max_output_buffer(source_sptr self, int port, long max_output_buffer)
        """
        return _audio_swig.source_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(source_sptr self, int i) -> long"""
        return _audio_swig.source_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(source_sptr self, long min_output_buffer)
        set_min_output_buffer(source_sptr self, int port, long min_output_buffer)
        """
        return _audio_swig.source_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(source_sptr self) -> float"""
        return _audio_swig.source_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(source_sptr self) -> float"""
        return _audio_swig.source_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(source_sptr self) -> float"""
        return _audio_swig.source_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(source_sptr self) -> float"""
        return _audio_swig.source_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(source_sptr self) -> float"""
        return _audio_swig.source_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(source_sptr self) -> float"""
        return _audio_swig.source_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(source_sptr self, int which) -> float
        pc_input_buffers_full(source_sptr self) -> pmt_vector_float
        """
        return _audio_swig.source_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(source_sptr self, int which) -> float
        pc_input_buffers_full_avg(source_sptr self) -> pmt_vector_float
        """
        return _audio_swig.source_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(source_sptr self, int which) -> float
        pc_input_buffers_full_var(source_sptr self) -> pmt_vector_float
        """
        return _audio_swig.source_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(source_sptr self, int which) -> float
        pc_output_buffers_full(source_sptr self) -> pmt_vector_float
        """
        return _audio_swig.source_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(source_sptr self, int which) -> float
        pc_output_buffers_full_avg(source_sptr self) -> pmt_vector_float
        """
        return _audio_swig.source_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(source_sptr self, int which) -> float
        pc_output_buffers_full_var(source_sptr self) -> pmt_vector_float
        """
        return _audio_swig.source_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(source_sptr self) -> float"""
        return _audio_swig.source_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(source_sptr self) -> float"""
        return _audio_swig.source_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(source_sptr self) -> float"""
        return _audio_swig.source_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(source_sptr self) -> float"""
        return _audio_swig.source_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(source_sptr self) -> float"""
        return _audio_swig.source_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(source_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _audio_swig.source_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(source_sptr self)"""
        return _audio_swig.source_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(source_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _audio_swig.source_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(source_sptr self) -> int"""
        return _audio_swig.source_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(source_sptr self) -> int"""
        return _audio_swig.source_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(source_sptr self, int priority) -> int"""
        return _audio_swig.source_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(source_sptr self) -> std::string"""
        return _audio_swig.source_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(source_sptr self) -> std::string"""
        return _audio_swig.source_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(source_sptr self) -> io_signature_sptr"""
        return _audio_swig.source_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(source_sptr self) -> io_signature_sptr"""
        return _audio_swig.source_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(source_sptr self) -> long"""
        return _audio_swig.source_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(source_sptr self) -> basic_block_sptr"""
        return _audio_swig.source_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(source_sptr self, int ninputs, int noutputs) -> bool"""
        return _audio_swig.source_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(source_sptr self) -> std::string"""
        return _audio_swig.source_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(source_sptr self, std::string name)"""
        return _audio_swig.source_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(source_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _audio_swig.source_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(source_sptr self) -> swig_pmt_ptr"""
        return _audio_swig.source_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(source_sptr self) -> swig_pmt_ptr"""
        return _audio_swig.source_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(source_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _audio_swig.source_sptr_message_subscribers(self, which_port)

# Register source_sptr in _audio_swig:
_audio_swig.source_sptr_swigregister(source_sptr)


source_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
source = source.make;

class sink_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::audio::sink > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(sink_sptr self) -> sink_sptr
        __init__(sink_sptr self, sink p) -> sink_sptr
        """
        _audio_swig.sink_sptr_swiginit(self, _audio_swig.new_sink_sptr(*args))

    def __deref__(self) -> "gr::audio::sink *":
        r"""__deref__(sink_sptr self) -> sink"""
        return _audio_swig.sink_sptr___deref__(self)
    __swig_destroy__ = _audio_swig.delete_sink_sptr

    def make(self, *args, **kwargs) -> "gr::audio::sink::sptr":
        r"""make(sink_sptr self, int sampling_rate, std::string const device_name="", bool ok_to_block=True) -> sink_sptr"""
        return _audio_swig.sink_sptr_make(self, *args, **kwargs)

    def history(self) -> "unsigned int":
        r"""history(sink_sptr self) -> unsigned int"""
        return _audio_swig.sink_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(sink_sptr self, int which, int delay)
        declare_sample_delay(sink_sptr self, unsigned int delay)
        """
        return _audio_swig.sink_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(sink_sptr self, int which) -> unsigned int"""
        return _audio_swig.sink_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(sink_sptr self, int multiple)"""
        return _audio_swig.sink_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(sink_sptr self) -> int"""
        return _audio_swig.sink_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(sink_sptr self) -> double"""
        return _audio_swig.sink_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(sink_sptr self) -> uint64_t"""
        return _audio_swig.sink_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(sink_sptr self) -> uint64_t"""
        return _audio_swig.sink_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(sink_sptr self) -> bool"""
        return _audio_swig.sink_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(sink_sptr self) -> bool"""
        return _audio_swig.sink_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(sink_sptr self, unsigned int which_input) -> uint64_t"""
        return _audio_swig.sink_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(sink_sptr self, unsigned int which_output) -> uint64_t"""
        return _audio_swig.sink_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(sink_sptr self, std::string level)"""
        return _audio_swig.sink_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(sink_sptr self) -> std::string"""
        return _audio_swig.sink_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(sink_sptr self) -> int"""
        return _audio_swig.sink_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(sink_sptr self, int m)"""
        return _audio_swig.sink_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(sink_sptr self)"""
        return _audio_swig.sink_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(sink_sptr self) -> bool"""
        return _audio_swig.sink_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(sink_sptr self, int m)"""
        return _audio_swig.sink_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(sink_sptr self) -> int"""
        return _audio_swig.sink_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(sink_sptr self, int i) -> long"""
        return _audio_swig.sink_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(sink_sptr self, long max_output_buffer)
        set_max_output_buffer(sink_sptr self, int port, long max_output_buffer)
        """
        return _audio_swig.sink_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(sink_sptr self, int i) -> long"""
        return _audio_swig.sink_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(sink_sptr self, long min_output_buffer)
        set_min_output_buffer(sink_sptr self, int port, long min_output_buffer)
        """
        return _audio_swig.sink_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(sink_sptr self) -> float"""
        return _audio_swig.sink_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(sink_sptr self) -> float"""
        return _audio_swig.sink_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(sink_sptr self) -> float"""
        return _audio_swig.sink_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(sink_sptr self) -> float"""
        return _audio_swig.sink_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(sink_sptr self) -> float"""
        return _audio_swig.sink_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(sink_sptr self) -> float"""
        return _audio_swig.sink_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(sink_sptr self, int which) -> float
        pc_input_buffers_full(sink_sptr self) -> pmt_vector_float
        """
        return _audio_swig.sink_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(sink_sptr self, int which) -> float
        pc_input_buffers_full_avg(sink_sptr self) -> pmt_vector_float
        """
        return _audio_swig.sink_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(sink_sptr self, int which) -> float
        pc_input_buffers_full_var(sink_sptr self) -> pmt_vector_float
        """
        return _audio_swig.sink_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(sink_sptr self, int which) -> float
        pc_output_buffers_full(sink_sptr self) -> pmt_vector_float
        """
        return _audio_swig.sink_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(sink_sptr self, int which) -> float
        pc_output_buffers_full_avg(sink_sptr self) -> pmt_vector_float
        """
        return _audio_swig.sink_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(sink_sptr self, int which) -> float
        pc_output_buffers_full_var(sink_sptr self) -> pmt_vector_float
        """
        return _audio_swig.sink_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(sink_sptr self) -> float"""
        return _audio_swig.sink_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(sink_sptr self) -> float"""
        return _audio_swig.sink_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(sink_sptr self) -> float"""
        return _audio_swig.sink_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(sink_sptr self) -> float"""
        return _audio_swig.sink_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(sink_sptr self) -> float"""
        return _audio_swig.sink_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(sink_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _audio_swig.sink_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(sink_sptr self)"""
        return _audio_swig.sink_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(sink_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _audio_swig.sink_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(sink_sptr self) -> int"""
        return _audio_swig.sink_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(sink_sptr self) -> int"""
        return _audio_swig.sink_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(sink_sptr self, int priority) -> int"""
        return _audio_swig.sink_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(sink_sptr self) -> std::string"""
        return _audio_swig.sink_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(sink_sptr self) -> std::string"""
        return _audio_swig.sink_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(sink_sptr self) -> io_signature_sptr"""
        return _audio_swig.sink_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(sink_sptr self) -> io_signature_sptr"""
        return _audio_swig.sink_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(sink_sptr self) -> long"""
        return _audio_swig.sink_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(sink_sptr self) -> basic_block_sptr"""
        return _audio_swig.sink_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(sink_sptr self, int ninputs, int noutputs) -> bool"""
        return _audio_swig.sink_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(sink_sptr self) -> std::string"""
        return _audio_swig.sink_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(sink_sptr self, std::string name)"""
        return _audio_swig.sink_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(sink_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _audio_swig.sink_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(sink_sptr self) -> swig_pmt_ptr"""
        return _audio_swig.sink_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(sink_sptr self) -> swig_pmt_ptr"""
        return _audio_swig.sink_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(sink_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _audio_swig.sink_sptr_message_subscribers(self, which_port)

# Register sink_sptr in _audio_swig:
_audio_swig.sink_sptr_swigregister(sink_sptr)


sink_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
sink = sink.make;






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.1
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _fft_swig
else:
    import _fft_swig

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def high_res_timer_now() -> "gr::high_res_timer_type":
    r"""high_res_timer_now() -> gr::high_res_timer_type"""
    return _fft_swig.high_res_timer_now()

def high_res_timer_now_perfmon() -> "gr::high_res_timer_type":
    r"""high_res_timer_now_perfmon() -> gr::high_res_timer_type"""
    return _fft_swig.high_res_timer_now_perfmon()

def high_res_timer_tps() -> "gr::high_res_timer_type":
    r"""high_res_timer_tps() -> gr::high_res_timer_type"""
    return _fft_swig.high_res_timer_tps()

def high_res_timer_epoch() -> "gr::high_res_timer_type":
    r"""high_res_timer_epoch() -> gr::high_res_timer_type"""
    return _fft_swig.high_res_timer_epoch()
class fft_vcc(object):
    r"""Proxy of C++ gr::fft::fft_vcc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def make(fft_size: "int", forward: "bool", window: "pmt_vector_float", shift: "bool"=False, nthreads: "int"=1) -> "gr::fft::fft_vcc::sptr":
        r"""make(int fft_size, bool forward, pmt_vector_float window, bool shift=False, int nthreads=1) -> fft_vcc_sptr"""
        return _fft_swig.fft_vcc_make(fft_size, forward, window, shift, nthreads)

    def set_nthreads(self, n: "int") -> "void":
        r"""set_nthreads(fft_vcc self, int n)"""
        return _fft_swig.fft_vcc_set_nthreads(self, n)

    def nthreads(self) -> "int":
        r"""nthreads(fft_vcc self) -> int"""
        return _fft_swig.fft_vcc_nthreads(self)

    def set_window(self, window: "pmt_vector_float") -> "bool":
        r"""set_window(fft_vcc self, pmt_vector_float window) -> bool"""
        return _fft_swig.fft_vcc_set_window(self, window)
    __swig_destroy__ = _fft_swig.delete_fft_vcc

# Register fft_vcc in _fft_swig:
_fft_swig.fft_vcc_swigregister(fft_vcc)

def fft_vcc_make(fft_size: "int", forward: "bool", window: "pmt_vector_float", shift: "bool"=False, nthreads: "int"=1) -> "gr::fft::fft_vcc::sptr":
    r"""fft_vcc_make(int fft_size, bool forward, pmt_vector_float window, bool shift=False, int nthreads=1) -> fft_vcc_sptr"""
    return _fft_swig.fft_vcc_make(fft_size, forward, window, shift, nthreads)

class fft_vfc(object):
    r"""Proxy of C++ gr::fft::fft_vfc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def make(fft_size: "int", forward: "bool", window: "pmt_vector_float", nthreads: "int"=1) -> "gr::fft::fft_vfc::sptr":
        r"""make(int fft_size, bool forward, pmt_vector_float window, int nthreads=1) -> fft_vfc_sptr"""
        return _fft_swig.fft_vfc_make(fft_size, forward, window, nthreads)

    def set_nthreads(self, n: "int") -> "void":
        r"""set_nthreads(fft_vfc self, int n)"""
        return _fft_swig.fft_vfc_set_nthreads(self, n)

    def nthreads(self) -> "int":
        r"""nthreads(fft_vfc self) -> int"""
        return _fft_swig.fft_vfc_nthreads(self)

    def set_window(self, window: "pmt_vector_float") -> "bool":
        r"""set_window(fft_vfc self, pmt_vector_float window) -> bool"""
        return _fft_swig.fft_vfc_set_window(self, window)
    __swig_destroy__ = _fft_swig.delete_fft_vfc

# Register fft_vfc in _fft_swig:
_fft_swig.fft_vfc_swigregister(fft_vfc)

def fft_vfc_make(fft_size: "int", forward: "bool", window: "pmt_vector_float", nthreads: "int"=1) -> "gr::fft::fft_vfc::sptr":
    r"""fft_vfc_make(int fft_size, bool forward, pmt_vector_float window, int nthreads=1) -> fft_vfc_sptr"""
    return _fft_swig.fft_vfc_make(fft_size, forward, window, nthreads)

class goertzel_fc(object):
    r"""Proxy of C++ gr::fft::goertzel_fc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def make(rate: "int", len: "int", freq: "float") -> "gr::fft::goertzel_fc::sptr":
        r"""make(int rate, int len, float freq) -> goertzel_fc_sptr"""
        return _fft_swig.goertzel_fc_make(rate, len, freq)

    def set_freq(self, freq: "float") -> "void":
        r"""set_freq(goertzel_fc self, float freq)"""
        return _fft_swig.goertzel_fc_set_freq(self, freq)

    def set_rate(self, rate: "int") -> "void":
        r"""set_rate(goertzel_fc self, int rate)"""
        return _fft_swig.goertzel_fc_set_rate(self, rate)

    def freq(self) -> "float":
        r"""freq(goertzel_fc self) -> float"""
        return _fft_swig.goertzel_fc_freq(self)

    def rate(self) -> "int":
        r"""rate(goertzel_fc self) -> int"""
        return _fft_swig.goertzel_fc_rate(self)
    __swig_destroy__ = _fft_swig.delete_goertzel_fc

# Register goertzel_fc in _fft_swig:
_fft_swig.goertzel_fc_swigregister(goertzel_fc)

def goertzel_fc_make(rate: "int", len: "int", freq: "float") -> "gr::fft::goertzel_fc::sptr":
    r"""goertzel_fc_make(int rate, int len, float freq) -> goertzel_fc_sptr"""
    return _fft_swig.goertzel_fc_make(rate, len, freq)

class window(object):
    r"""Proxy of C++ gr::fft::window class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    WIN_HAMMING = _fft_swig.window_WIN_HAMMING
    
    WIN_HANN = _fft_swig.window_WIN_HANN
    
    WIN_BLACKMAN = _fft_swig.window_WIN_BLACKMAN
    
    WIN_RECTANGULAR = _fft_swig.window_WIN_RECTANGULAR
    
    WIN_KAISER = _fft_swig.window_WIN_KAISER
    
    WIN_BLACKMAN_hARRIS = _fft_swig.window_WIN_BLACKMAN_hARRIS
    
    WIN_BLACKMAN_HARRIS = _fft_swig.window_WIN_BLACKMAN_HARRIS
    
    WIN_BARTLETT = _fft_swig.window_WIN_BARTLETT
    
    WIN_FLATTOP = _fft_swig.window_WIN_FLATTOP
    

    @staticmethod
    def max_attenuation(type: "gr::fft::window::win_type", beta: "double"=6.76) -> "double":
        r"""max_attenuation(gr::fft::window::win_type type, double beta=6.76) -> double"""
        return _fft_swig.window_max_attenuation(type, beta)

    @staticmethod
    def coswindow(*args) -> "std::vector< float,std::allocator< float > >":
        r"""
        coswindow(int ntaps, float c0, float c1, float c2) -> pmt_vector_float
        coswindow(int ntaps, float c0, float c1, float c2, float c3) -> pmt_vector_float
        coswindow(int ntaps, float c0, float c1, float c2, float c3, float c4) -> pmt_vector_float
        """
        return _fft_swig.window_coswindow(*args)

    @staticmethod
    def rectangular(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""rectangular(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_rectangular(ntaps)

    @staticmethod
    def hamming(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""hamming(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_hamming(ntaps)

    @staticmethod
    def hann(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""hann(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_hann(ntaps)

    @staticmethod
    def hanning(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""hanning(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_hanning(ntaps)

    @staticmethod
    def blackman(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""blackman(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_blackman(ntaps)

    @staticmethod
    def blackman2(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""blackman2(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_blackman2(ntaps)

    @staticmethod
    def blackman3(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""blackman3(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_blackman3(ntaps)

    @staticmethod
    def blackman4(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""blackman4(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_blackman4(ntaps)

    @staticmethod
    def blackman_harris(ntaps: "int", atten: "int"=92) -> "std::vector< float,std::allocator< float > >":
        r"""blackman_harris(int ntaps, int atten=92) -> pmt_vector_float"""
        return _fft_swig.window_blackman_harris(ntaps, atten)

    @staticmethod
    def blackmanharris(ntaps: "int", atten: "int"=92) -> "std::vector< float,std::allocator< float > >":
        r"""blackmanharris(int ntaps, int atten=92) -> pmt_vector_float"""
        return _fft_swig.window_blackmanharris(ntaps, atten)

    @staticmethod
    def nuttall(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""nuttall(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_nuttall(ntaps)

    @staticmethod
    def nuttal(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""nuttal(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_nuttal(ntaps)

    @staticmethod
    def blackman_nuttall(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""blackman_nuttall(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_blackman_nuttall(ntaps)

    @staticmethod
    def blackman_nuttal(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""blackman_nuttal(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_blackman_nuttal(ntaps)

    @staticmethod
    def nuttall_cfd(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""nuttall_cfd(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_nuttall_cfd(ntaps)

    @staticmethod
    def nuttal_cfd(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""nuttal_cfd(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_nuttal_cfd(ntaps)

    @staticmethod
    def flattop(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""flattop(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_flattop(ntaps)

    @staticmethod
    def kaiser(ntaps: "int", beta: "double") -> "std::vector< float,std::allocator< float > >":
        r"""kaiser(int ntaps, double beta) -> pmt_vector_float"""
        return _fft_swig.window_kaiser(ntaps, beta)

    @staticmethod
    def bartlett(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""bartlett(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_bartlett(ntaps)

    @staticmethod
    def welch(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""welch(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_welch(ntaps)

    @staticmethod
    def parzen(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""parzen(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_parzen(ntaps)

    @staticmethod
    def exponential(ntaps: "int", d: "double") -> "std::vector< float,std::allocator< float > >":
        r"""exponential(int ntaps, double d) -> pmt_vector_float"""
        return _fft_swig.window_exponential(ntaps, d)

    @staticmethod
    def riemann(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
        r"""riemann(int ntaps) -> pmt_vector_float"""
        return _fft_swig.window_riemann(ntaps)

    @staticmethod
    def build(type: "gr::fft::window::win_type", ntaps: "int", beta: "double") -> "std::vector< float,std::allocator< float > >":
        r"""build(gr::fft::window::win_type type, int ntaps, double beta) -> pmt_vector_float"""
        return _fft_swig.window_build(type, ntaps, beta)

    def __init__(self):
        r"""__init__(window self) -> window"""
        _fft_swig.window_swiginit(self, _fft_swig.new_window())
    __swig_destroy__ = _fft_swig.delete_window

# Register window in _fft_swig:
_fft_swig.window_swigregister(window)

def window_max_attenuation(type: "gr::fft::window::win_type", beta: "double"=6.76) -> "double":
    r"""window_max_attenuation(gr::fft::window::win_type type, double beta=6.76) -> double"""
    return _fft_swig.window_max_attenuation(type, beta)

def window_coswindow(*args) -> "std::vector< float,std::allocator< float > >":
    r"""
    window_coswindow(int ntaps, float c0, float c1, float c2) -> pmt_vector_float
    window_coswindow(int ntaps, float c0, float c1, float c2, float c3) -> pmt_vector_float
    window_coswindow(int ntaps, float c0, float c1, float c2, float c3, float c4) -> pmt_vector_float
    """
    return _fft_swig.window_coswindow(*args)

def window_rectangular(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_rectangular(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_rectangular(ntaps)

def window_hamming(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_hamming(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_hamming(ntaps)

def window_hann(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_hann(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_hann(ntaps)

def window_hanning(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_hanning(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_hanning(ntaps)

def window_blackman(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_blackman(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_blackman(ntaps)

def window_blackman2(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_blackman2(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_blackman2(ntaps)

def window_blackman3(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_blackman3(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_blackman3(ntaps)

def window_blackman4(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_blackman4(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_blackman4(ntaps)

def window_blackman_harris(ntaps: "int", atten: "int"=92) -> "std::vector< float,std::allocator< float > >":
    r"""window_blackman_harris(int ntaps, int atten=92) -> pmt_vector_float"""
    return _fft_swig.window_blackman_harris(ntaps, atten)

def window_blackmanharris(ntaps: "int", atten: "int"=92) -> "std::vector< float,std::allocator< float > >":
    r"""window_blackmanharris(int ntaps, int atten=92) -> pmt_vector_float"""
    return _fft_swig.window_blackmanharris(ntaps, atten)

def window_nuttall(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_nuttall(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_nuttall(ntaps)

def window_nuttal(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_nuttal(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_nuttal(ntaps)

def window_blackman_nuttall(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_blackman_nuttall(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_blackman_nuttall(ntaps)

def window_blackman_nuttal(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_blackman_nuttal(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_blackman_nuttal(ntaps)

def window_nuttall_cfd(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_nuttall_cfd(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_nuttall_cfd(ntaps)

def window_nuttal_cfd(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_nuttal_cfd(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_nuttal_cfd(ntaps)

def window_flattop(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_flattop(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_flattop(ntaps)

def window_kaiser(ntaps: "int", beta: "double") -> "std::vector< float,std::allocator< float > >":
    r"""window_kaiser(int ntaps, double beta) -> pmt_vector_float"""
    return _fft_swig.window_kaiser(ntaps, beta)

def window_bartlett(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_bartlett(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_bartlett(ntaps)

def window_welch(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_welch(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_welch(ntaps)

def window_parzen(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_parzen(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_parzen(ntaps)

def window_exponential(ntaps: "int", d: "double") -> "std::vector< float,std::allocator< float > >":
    r"""window_exponential(int ntaps, double d) -> pmt_vector_float"""
    return _fft_swig.window_exponential(ntaps, d)

def window_riemann(ntaps: "int") -> "std::vector< float,std::allocator< float > >":
    r"""window_riemann(int ntaps) -> pmt_vector_float"""
    return _fft_swig.window_riemann(ntaps)

def window_build(type: "gr::fft::window::win_type", ntaps: "int", beta: "double") -> "std::vector< float,std::allocator< float > >":
    r"""window_build(gr::fft::window::win_type type, int ntaps, double beta) -> pmt_vector_float"""
    return _fft_swig.window_build(type, ntaps, beta)

class fft_vcc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::fft::fft_vcc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(fft_vcc_sptr self) -> fft_vcc_sptr
        __init__(fft_vcc_sptr self, fft_vcc p) -> fft_vcc_sptr
        """
        _fft_swig.fft_vcc_sptr_swiginit(self, _fft_swig.new_fft_vcc_sptr(*args))

    def __deref__(self) -> "gr::fft::fft_vcc *":
        r"""__deref__(fft_vcc_sptr self) -> fft_vcc"""
        return _fft_swig.fft_vcc_sptr___deref__(self)
    __swig_destroy__ = _fft_swig.delete_fft_vcc_sptr

    def make(self, fft_size: "int", forward: "bool", window: "pmt_vector_float", shift: "bool"=False, nthreads: "int"=1) -> "gr::fft::fft_vcc::sptr":
        r"""make(fft_vcc_sptr self, int fft_size, bool forward, pmt_vector_float window, bool shift=False, int nthreads=1) -> fft_vcc_sptr"""
        return _fft_swig.fft_vcc_sptr_make(self, fft_size, forward, window, shift, nthreads)

    def set_nthreads(self, n: "int") -> "void":
        r"""set_nthreads(fft_vcc_sptr self, int n)"""
        return _fft_swig.fft_vcc_sptr_set_nthreads(self, n)

    def nthreads(self) -> "int":
        r"""nthreads(fft_vcc_sptr self) -> int"""
        return _fft_swig.fft_vcc_sptr_nthreads(self)

    def set_window(self, window: "pmt_vector_float") -> "bool":
        r"""set_window(fft_vcc_sptr self, pmt_vector_float window) -> bool"""
        return _fft_swig.fft_vcc_sptr_set_window(self, window)

    def history(self) -> "unsigned int":
        r"""history(fft_vcc_sptr self) -> unsigned int"""
        return _fft_swig.fft_vcc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(fft_vcc_sptr self, int which, int delay)
        declare_sample_delay(fft_vcc_sptr self, unsigned int delay)
        """
        return _fft_swig.fft_vcc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(fft_vcc_sptr self, int which) -> unsigned int"""
        return _fft_swig.fft_vcc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(fft_vcc_sptr self, int multiple)"""
        return _fft_swig.fft_vcc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(fft_vcc_sptr self) -> int"""
        return _fft_swig.fft_vcc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(fft_vcc_sptr self) -> double"""
        return _fft_swig.fft_vcc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(fft_vcc_sptr self) -> uint64_t"""
        return _fft_swig.fft_vcc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(fft_vcc_sptr self) -> uint64_t"""
        return _fft_swig.fft_vcc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(fft_vcc_sptr self) -> bool"""
        return _fft_swig.fft_vcc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(fft_vcc_sptr self) -> bool"""
        return _fft_swig.fft_vcc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(fft_vcc_sptr self, unsigned int which_input) -> uint64_t"""
        return _fft_swig.fft_vcc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(fft_vcc_sptr self, unsigned int which_output) -> uint64_t"""
        return _fft_swig.fft_vcc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(fft_vcc_sptr self, std::string level)"""
        return _fft_swig.fft_vcc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(fft_vcc_sptr self) -> std::string"""
        return _fft_swig.fft_vcc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(fft_vcc_sptr self) -> int"""
        return _fft_swig.fft_vcc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(fft_vcc_sptr self, int m)"""
        return _fft_swig.fft_vcc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(fft_vcc_sptr self)"""
        return _fft_swig.fft_vcc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(fft_vcc_sptr self) -> bool"""
        return _fft_swig.fft_vcc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(fft_vcc_sptr self, int m)"""
        return _fft_swig.fft_vcc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(fft_vcc_sptr self) -> int"""
        return _fft_swig.fft_vcc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(fft_vcc_sptr self, int i) -> long"""
        return _fft_swig.fft_vcc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(fft_vcc_sptr self, long max_output_buffer)
        set_max_output_buffer(fft_vcc_sptr self, int port, long max_output_buffer)
        """
        return _fft_swig.fft_vcc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(fft_vcc_sptr self, int i) -> long"""
        return _fft_swig.fft_vcc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(fft_vcc_sptr self, long min_output_buffer)
        set_min_output_buffer(fft_vcc_sptr self, int port, long min_output_buffer)
        """
        return _fft_swig.fft_vcc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(fft_vcc_sptr self) -> float"""
        return _fft_swig.fft_vcc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(fft_vcc_sptr self) -> float"""
        return _fft_swig.fft_vcc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(fft_vcc_sptr self) -> float"""
        return _fft_swig.fft_vcc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(fft_vcc_sptr self) -> float"""
        return _fft_swig.fft_vcc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(fft_vcc_sptr self) -> float"""
        return _fft_swig.fft_vcc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(fft_vcc_sptr self) -> float"""
        return _fft_swig.fft_vcc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(fft_vcc_sptr self, int which) -> float
        pc_input_buffers_full(fft_vcc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vcc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(fft_vcc_sptr self, int which) -> float
        pc_input_buffers_full_avg(fft_vcc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vcc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(fft_vcc_sptr self, int which) -> float
        pc_input_buffers_full_var(fft_vcc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vcc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(fft_vcc_sptr self, int which) -> float
        pc_output_buffers_full(fft_vcc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vcc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(fft_vcc_sptr self, int which) -> float
        pc_output_buffers_full_avg(fft_vcc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vcc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(fft_vcc_sptr self, int which) -> float
        pc_output_buffers_full_var(fft_vcc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vcc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(fft_vcc_sptr self) -> float"""
        return _fft_swig.fft_vcc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(fft_vcc_sptr self) -> float"""
        return _fft_swig.fft_vcc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(fft_vcc_sptr self) -> float"""
        return _fft_swig.fft_vcc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(fft_vcc_sptr self) -> float"""
        return _fft_swig.fft_vcc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(fft_vcc_sptr self) -> float"""
        return _fft_swig.fft_vcc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(fft_vcc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _fft_swig.fft_vcc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(fft_vcc_sptr self)"""
        return _fft_swig.fft_vcc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(fft_vcc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _fft_swig.fft_vcc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(fft_vcc_sptr self) -> int"""
        return _fft_swig.fft_vcc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(fft_vcc_sptr self) -> int"""
        return _fft_swig.fft_vcc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(fft_vcc_sptr self, int priority) -> int"""
        return _fft_swig.fft_vcc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(fft_vcc_sptr self) -> std::string"""
        return _fft_swig.fft_vcc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(fft_vcc_sptr self) -> std::string"""
        return _fft_swig.fft_vcc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(fft_vcc_sptr self) -> io_signature_sptr"""
        return _fft_swig.fft_vcc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(fft_vcc_sptr self) -> io_signature_sptr"""
        return _fft_swig.fft_vcc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(fft_vcc_sptr self) -> long"""
        return _fft_swig.fft_vcc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(fft_vcc_sptr self) -> basic_block_sptr"""
        return _fft_swig.fft_vcc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(fft_vcc_sptr self, int ninputs, int noutputs) -> bool"""
        return _fft_swig.fft_vcc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(fft_vcc_sptr self) -> std::string"""
        return _fft_swig.fft_vcc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(fft_vcc_sptr self, std::string name)"""
        return _fft_swig.fft_vcc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(fft_vcc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _fft_swig.fft_vcc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(fft_vcc_sptr self) -> swig_pmt_ptr"""
        return _fft_swig.fft_vcc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(fft_vcc_sptr self) -> swig_pmt_ptr"""
        return _fft_swig.fft_vcc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(fft_vcc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _fft_swig.fft_vcc_sptr_message_subscribers(self, which_port)

# Register fft_vcc_sptr in _fft_swig:
_fft_swig.fft_vcc_sptr_swigregister(fft_vcc_sptr)


fft_vcc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
fft_vcc = fft_vcc.make;

class fft_vfc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::fft::fft_vfc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(fft_vfc_sptr self) -> fft_vfc_sptr
        __init__(fft_vfc_sptr self, fft_vfc p) -> fft_vfc_sptr
        """
        _fft_swig.fft_vfc_sptr_swiginit(self, _fft_swig.new_fft_vfc_sptr(*args))

    def __deref__(self) -> "gr::fft::fft_vfc *":
        r"""__deref__(fft_vfc_sptr self) -> fft_vfc"""
        return _fft_swig.fft_vfc_sptr___deref__(self)
    __swig_destroy__ = _fft_swig.delete_fft_vfc_sptr

    def make(self, fft_size: "int", forward: "bool", window: "pmt_vector_float", nthreads: "int"=1) -> "gr::fft::fft_vfc::sptr":
        r"""make(fft_vfc_sptr self, int fft_size, bool forward, pmt_vector_float window, int nthreads=1) -> fft_vfc_sptr"""
        return _fft_swig.fft_vfc_sptr_make(self, fft_size, forward, window, nthreads)

    def set_nthreads(self, n: "int") -> "void":
        r"""set_nthreads(fft_vfc_sptr self, int n)"""
        return _fft_swig.fft_vfc_sptr_set_nthreads(self, n)

    def nthreads(self) -> "int":
        r"""nthreads(fft_vfc_sptr self) -> int"""
        return _fft_swig.fft_vfc_sptr_nthreads(self)

    def set_window(self, window: "pmt_vector_float") -> "bool":
        r"""set_window(fft_vfc_sptr self, pmt_vector_float window) -> bool"""
        return _fft_swig.fft_vfc_sptr_set_window(self, window)

    def history(self) -> "unsigned int":
        r"""history(fft_vfc_sptr self) -> unsigned int"""
        return _fft_swig.fft_vfc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(fft_vfc_sptr self, int which, int delay)
        declare_sample_delay(fft_vfc_sptr self, unsigned int delay)
        """
        return _fft_swig.fft_vfc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(fft_vfc_sptr self, int which) -> unsigned int"""
        return _fft_swig.fft_vfc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(fft_vfc_sptr self, int multiple)"""
        return _fft_swig.fft_vfc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(fft_vfc_sptr self) -> int"""
        return _fft_swig.fft_vfc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(fft_vfc_sptr self) -> double"""
        return _fft_swig.fft_vfc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(fft_vfc_sptr self) -> uint64_t"""
        return _fft_swig.fft_vfc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(fft_vfc_sptr self) -> uint64_t"""
        return _fft_swig.fft_vfc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(fft_vfc_sptr self) -> bool"""
        return _fft_swig.fft_vfc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(fft_vfc_sptr self) -> bool"""
        return _fft_swig.fft_vfc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(fft_vfc_sptr self, unsigned int which_input) -> uint64_t"""
        return _fft_swig.fft_vfc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(fft_vfc_sptr self, unsigned int which_output) -> uint64_t"""
        return _fft_swig.fft_vfc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(fft_vfc_sptr self, std::string level)"""
        return _fft_swig.fft_vfc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(fft_vfc_sptr self) -> std::string"""
        return _fft_swig.fft_vfc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(fft_vfc_sptr self) -> int"""
        return _fft_swig.fft_vfc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(fft_vfc_sptr self, int m)"""
        return _fft_swig.fft_vfc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(fft_vfc_sptr self)"""
        return _fft_swig.fft_vfc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(fft_vfc_sptr self) -> bool"""
        return _fft_swig.fft_vfc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(fft_vfc_sptr self, int m)"""
        return _fft_swig.fft_vfc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(fft_vfc_sptr self) -> int"""
        return _fft_swig.fft_vfc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(fft_vfc_sptr self, int i) -> long"""
        return _fft_swig.fft_vfc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(fft_vfc_sptr self, long max_output_buffer)
        set_max_output_buffer(fft_vfc_sptr self, int port, long max_output_buffer)
        """
        return _fft_swig.fft_vfc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(fft_vfc_sptr self, int i) -> long"""
        return _fft_swig.fft_vfc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(fft_vfc_sptr self, long min_output_buffer)
        set_min_output_buffer(fft_vfc_sptr self, int port, long min_output_buffer)
        """
        return _fft_swig.fft_vfc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(fft_vfc_sptr self) -> float"""
        return _fft_swig.fft_vfc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(fft_vfc_sptr self) -> float"""
        return _fft_swig.fft_vfc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(fft_vfc_sptr self) -> float"""
        return _fft_swig.fft_vfc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(fft_vfc_sptr self) -> float"""
        return _fft_swig.fft_vfc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(fft_vfc_sptr self) -> float"""
        return _fft_swig.fft_vfc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(fft_vfc_sptr self) -> float"""
        return _fft_swig.fft_vfc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(fft_vfc_sptr self, int which) -> float
        pc_input_buffers_full(fft_vfc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vfc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(fft_vfc_sptr self, int which) -> float
        pc_input_buffers_full_avg(fft_vfc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vfc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(fft_vfc_sptr self, int which) -> float
        pc_input_buffers_full_var(fft_vfc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vfc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(fft_vfc_sptr self, int which) -> float
        pc_output_buffers_full(fft_vfc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vfc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(fft_vfc_sptr self, int which) -> float
        pc_output_buffers_full_avg(fft_vfc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vfc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(fft_vfc_sptr self, int which) -> float
        pc_output_buffers_full_var(fft_vfc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.fft_vfc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(fft_vfc_sptr self) -> float"""
        return _fft_swig.fft_vfc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(fft_vfc_sptr self) -> float"""
        return _fft_swig.fft_vfc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(fft_vfc_sptr self) -> float"""
        return _fft_swig.fft_vfc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(fft_vfc_sptr self) -> float"""
        return _fft_swig.fft_vfc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(fft_vfc_sptr self) -> float"""
        return _fft_swig.fft_vfc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(fft_vfc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _fft_swig.fft_vfc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(fft_vfc_sptr self)"""
        return _fft_swig.fft_vfc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(fft_vfc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _fft_swig.fft_vfc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(fft_vfc_sptr self) -> int"""
        return _fft_swig.fft_vfc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(fft_vfc_sptr self) -> int"""
        return _fft_swig.fft_vfc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(fft_vfc_sptr self, int priority) -> int"""
        return _fft_swig.fft_vfc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(fft_vfc_sptr self) -> std::string"""
        return _fft_swig.fft_vfc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(fft_vfc_sptr self) -> std::string"""
        return _fft_swig.fft_vfc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(fft_vfc_sptr self) -> io_signature_sptr"""
        return _fft_swig.fft_vfc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(fft_vfc_sptr self) -> io_signature_sptr"""
        return _fft_swig.fft_vfc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(fft_vfc_sptr self) -> long"""
        return _fft_swig.fft_vfc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(fft_vfc_sptr self) -> basic_block_sptr"""
        return _fft_swig.fft_vfc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(fft_vfc_sptr self, int ninputs, int noutputs) -> bool"""
        return _fft_swig.fft_vfc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(fft_vfc_sptr self) -> std::string"""
        return _fft_swig.fft_vfc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(fft_vfc_sptr self, std::string name)"""
        return _fft_swig.fft_vfc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(fft_vfc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _fft_swig.fft_vfc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(fft_vfc_sptr self) -> swig_pmt_ptr"""
        return _fft_swig.fft_vfc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(fft_vfc_sptr self) -> swig_pmt_ptr"""
        return _fft_swig.fft_vfc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(fft_vfc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _fft_swig.fft_vfc_sptr_message_subscribers(self, which_port)

# Register fft_vfc_sptr in _fft_swig:
_fft_swig.fft_vfc_sptr_swigregister(fft_vfc_sptr)


fft_vfc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
fft_vfc = fft_vfc.make;

class goertzel_fc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::fft::goertzel_fc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(goertzel_fc_sptr self) -> goertzel_fc_sptr
        __init__(goertzel_fc_sptr self, goertzel_fc p) -> goertzel_fc_sptr
        """
        _fft_swig.goertzel_fc_sptr_swiginit(self, _fft_swig.new_goertzel_fc_sptr(*args))

    def __deref__(self) -> "gr::fft::goertzel_fc *":
        r"""__deref__(goertzel_fc_sptr self) -> goertzel_fc"""
        return _fft_swig.goertzel_fc_sptr___deref__(self)
    __swig_destroy__ = _fft_swig.delete_goertzel_fc_sptr

    def make(self, rate: "int", len: "int", freq: "float") -> "gr::fft::goertzel_fc::sptr":
        r"""make(goertzel_fc_sptr self, int rate, int len, float freq) -> goertzel_fc_sptr"""
        return _fft_swig.goertzel_fc_sptr_make(self, rate, len, freq)

    def set_freq(self, freq: "float") -> "void":
        r"""set_freq(goertzel_fc_sptr self, float freq)"""
        return _fft_swig.goertzel_fc_sptr_set_freq(self, freq)

    def set_rate(self, rate: "int") -> "void":
        r"""set_rate(goertzel_fc_sptr self, int rate)"""
        return _fft_swig.goertzel_fc_sptr_set_rate(self, rate)

    def freq(self) -> "float":
        r"""freq(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_freq(self)

    def rate(self) -> "int":
        r"""rate(goertzel_fc_sptr self) -> int"""
        return _fft_swig.goertzel_fc_sptr_rate(self)

    def history(self) -> "unsigned int":
        r"""history(goertzel_fc_sptr self) -> unsigned int"""
        return _fft_swig.goertzel_fc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(goertzel_fc_sptr self, int which, int delay)
        declare_sample_delay(goertzel_fc_sptr self, unsigned int delay)
        """
        return _fft_swig.goertzel_fc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(goertzel_fc_sptr self, int which) -> unsigned int"""
        return _fft_swig.goertzel_fc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(goertzel_fc_sptr self, int multiple)"""
        return _fft_swig.goertzel_fc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(goertzel_fc_sptr self) -> int"""
        return _fft_swig.goertzel_fc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(goertzel_fc_sptr self) -> double"""
        return _fft_swig.goertzel_fc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(goertzel_fc_sptr self) -> uint64_t"""
        return _fft_swig.goertzel_fc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(goertzel_fc_sptr self) -> uint64_t"""
        return _fft_swig.goertzel_fc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(goertzel_fc_sptr self) -> bool"""
        return _fft_swig.goertzel_fc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(goertzel_fc_sptr self) -> bool"""
        return _fft_swig.goertzel_fc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(goertzel_fc_sptr self, unsigned int which_input) -> uint64_t"""
        return _fft_swig.goertzel_fc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(goertzel_fc_sptr self, unsigned int which_output) -> uint64_t"""
        return _fft_swig.goertzel_fc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(goertzel_fc_sptr self, std::string level)"""
        return _fft_swig.goertzel_fc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(goertzel_fc_sptr self) -> std::string"""
        return _fft_swig.goertzel_fc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(goertzel_fc_sptr self) -> int"""
        return _fft_swig.goertzel_fc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(goertzel_fc_sptr self, int m)"""
        return _fft_swig.goertzel_fc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(goertzel_fc_sptr self)"""
        return _fft_swig.goertzel_fc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(goertzel_fc_sptr self) -> bool"""
        return _fft_swig.goertzel_fc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(goertzel_fc_sptr self, int m)"""
        return _fft_swig.goertzel_fc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(goertzel_fc_sptr self) -> int"""
        return _fft_swig.goertzel_fc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(goertzel_fc_sptr self, int i) -> long"""
        return _fft_swig.goertzel_fc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(goertzel_fc_sptr self, long max_output_buffer)
        set_max_output_buffer(goertzel_fc_sptr self, int port, long max_output_buffer)
        """
        return _fft_swig.goertzel_fc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(goertzel_fc_sptr self, int i) -> long"""
        return _fft_swig.goertzel_fc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(goertzel_fc_sptr self, long min_output_buffer)
        set_min_output_buffer(goertzel_fc_sptr self, int port, long min_output_buffer)
        """
        return _fft_swig.goertzel_fc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(goertzel_fc_sptr self, int which) -> float
        pc_input_buffers_full(goertzel_fc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.goertzel_fc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(goertzel_fc_sptr self, int which) -> float
        pc_input_buffers_full_avg(goertzel_fc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.goertzel_fc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(goertzel_fc_sptr self, int which) -> float
        pc_input_buffers_full_var(goertzel_fc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.goertzel_fc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(goertzel_fc_sptr self, int which) -> float
        pc_output_buffers_full(goertzel_fc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.goertzel_fc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(goertzel_fc_sptr self, int which) -> float
        pc_output_buffers_full_avg(goertzel_fc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.goertzel_fc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(goertzel_fc_sptr self, int which) -> float
        pc_output_buffers_full_var(goertzel_fc_sptr self) -> pmt_vector_float
        """
        return _fft_swig.goertzel_fc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(goertzel_fc_sptr self) -> float"""
        return _fft_swig.goertzel_fc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(goertzel_fc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _fft_swig.goertzel_fc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(goertzel_fc_sptr self)"""
        return _fft_swig.goertzel_fc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(goertzel_fc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _fft_swig.goertzel_fc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(goertzel_fc_sptr self) -> int"""
        return _fft_swig.goertzel_fc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(goertzel_fc_sptr self) -> int"""
        return _fft_swig.goertzel_fc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(goertzel_fc_sptr self, int priority) -> int"""
        return _fft_swig.goertzel_fc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(goertzel_fc_sptr self) -> std::string"""
        return _fft_swig.goertzel_fc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(goertzel_fc_sptr self) -> std::string"""
        return _fft_swig.goertzel_fc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(goertzel_fc_sptr self) -> io_signature_sptr"""
        return _fft_swig.goertzel_fc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(goertzel_fc_sptr self) -> io_signature_sptr"""
        return _fft_swig.goertzel_fc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(goertzel_fc_sptr self) -> long"""
        return _fft_swig.goertzel_fc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(goertzel_fc_sptr self) -> basic_block_sptr"""
        return _fft_swig.goertzel_fc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(goertzel_fc_sptr self, int ninputs, int noutputs) -> bool"""
        return _fft_swig.goertzel_fc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(goertzel_fc_sptr self) -> std::string"""
        return _fft_swig.goertzel_fc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(goertzel_fc_sptr self, std::string name)"""
        return _fft_swig.goertzel_fc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(goertzel_fc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _fft_swig.goertzel_fc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(goertzel_fc_sptr self) -> swig_pmt_ptr"""
        return _fft_swig.goertzel_fc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(goertzel_fc_sptr self) -> swig_pmt_ptr"""
        return _fft_swig.goertzel_fc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(goertzel_fc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _fft_swig.goertzel_fc_sptr_message_subscribers(self, which_port)

# Register goertzel_fc_sptr in _fft_swig:
_fft_swig.goertzel_fc_sptr_swigregister(goertzel_fc_sptr)


goertzel_fc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
goertzel_fc = goertzel_fc.make;






#!D:\bld\gnuradio_1587690700010\_h_env\python.exe

from __future__ import print_function
from __future__ import unicode_literals
from gnuradio import gr
from gnuradio import blocks
import argparse
from volk_test_funcs import *

######################################################################

def float_to_char(N):
    op = blocks.float_to_char()
    tb = helper(N, op, gr.sizeof_float, gr.sizeof_char, 1, 1)
    return tb

######################################################################

def float_to_int(N):
    op = blocks.float_to_int()
    tb = helper(N, op, gr.sizeof_float, gr.sizeof_int, 1, 1)
    return tb

######################################################################

def float_to_short(N):
    op = blocks.float_to_short()
    tb = helper(N, op, gr.sizeof_float, gr.sizeof_short, 1, 1)
    return tb

######################################################################

def short_to_float(N):
    op = blocks.short_to_float()
    tb = helper(N, op, gr.sizeof_short, gr.sizeof_float, 1, 1)
    return tb

######################################################################

def short_to_char(N):
    op = blocks.short_to_char()
    tb = helper(N, op, gr.sizeof_short, gr.sizeof_char, 1, 1)
    return tb

######################################################################

def char_to_short(N):
    op = blocks.char_to_short()
    tb = helper(N, op, gr.sizeof_char, gr.sizeof_short, 1, 1)

######################################################################

def char_to_float(N):
    op = blocks.char_to_float()
    tb = helper(N, op, gr.sizeof_char, gr.sizeof_float, 1, 1)

#####################################################################

def int_to_float(N):
    op = blocks.int_to_float()
    tb = helper(N, op, gr.sizeof_int, gr.sizeof_float, 1, 1)
    return tb

######################################################################

def complex_to_float(N):
    op = blocks.complex_to_float()
    tb = helper(N, op, gr.sizeof_gr_complex, gr.sizeof_float, 1, 2)
    return tb

######################################################################

def complex_to_real(N):
    op = blocks.complex_to_real()
    tb = helper(N, op, gr.sizeof_gr_complex, gr.sizeof_float, 1, 1)
    return tb

######################################################################

def complex_to_imag(N):
    op = blocks.complex_to_imag()
    tb = helper(N, op, gr.sizeof_gr_complex, gr.sizeof_float, 1, 1)
    return tb

######################################################################

def complex_to_mag(N):
    op = blocks.complex_to_mag()
    tb = helper(N, op, gr.sizeof_gr_complex, gr.sizeof_float, 1, 1)
    return tb

######################################################################

def complex_to_mag_squared(N):
    op = blocks.complex_to_mag_squared()
    tb = helper(N, op, gr.sizeof_gr_complex, gr.sizeof_float, 1, 1)
    return tb

######################################################################


def run_tests(func, N, iters):
    print("Running Test: {0}".format(func.__name__))
    try:
        tb = func(N)
        t = timeit(tb, iters)
        res = format_results(func.__name__, t)
        return res
    except AttributeError:
        print("\tCould not run test. Skipping.")
        return None

def main():
    avail_tests = [float_to_char,
                   float_to_int,
                   float_to_short,
                   short_to_float,
                   short_to_char,
                   char_to_short,
                   char_to_float,
                   int_to_float,
                   complex_to_float,
                   complex_to_real,
                   complex_to_imag,
                   complex_to_mag,
                   complex_to_mag_squared]

    desc='Time an operation to compare with other implementations. \
          This program runs a simple GNU Radio flowgraph to test a \
          particular math function, mostly to compare the  \
          Volk-optimized implementation versus a regular \
          implementation. The results are stored to an SQLite database \
          that can then be read by volk_plot.py to plot the differences.'
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument('-L', '--label', type=str,
                        required=True, default=None,
                        help='Label of database table [default: %(default)s]')
    parser.add_argument('-D', '--database', type=str,
                        default="volk_results.db",
                        help='Database file to store data in [default: %(default)s]')
    parser.add_argument('-N', '--nitems', type=float,
                        default=1e9,
                        help='Number of items per iterations [default: %(default)s]')
    parser.add_argument('-I', '--iterations', type=int,
                        default=20,
                        help='Number of iterations [default: %(default)s]')
    parser.add_argument('--tests', type=int, nargs='*',
                        choices=list(range(len(avail_tests))),
                        help='A list of tests to run; can be a single test or a \
                              space-separated list.')
    parser.add_argument('--list', action='store_true',
                        help='List the available tests')
    parser.add_argument('--all', action='store_true',
                        help='Run all tests')
    args = parser.parse_args()

    if(args.list):
        print("Available Tests to Run:")
        print("\n".join(["\t{0}: {1}".format(i,f.__name__) for i,f in enumerate(avail_tests)]))
        sys.exit(0)

    N = int(args.nitems)
    iters = args.iterations
    label = args.label

    conn = create_connection(args.database)
    new_table(conn, label)

    if args.all:
        tests = list(range(len(avail_tests)))
    else:
        tests = args.tests

    for test in tests:
        res = run_tests(avail_tests[test], N, iters)
        if res is not None:
            replace_results(conn, label, N, iters, res)

if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        pass



#!D:\bld\gnuradio_1587690700010\_h_env\python.exe
#
# Copyright 2007,2008,2011 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#

from __future__ import print_function

import numpy

try:
    from pylab import *
except ImportError:
    print("Please install Python Matplotlib (http://matplotlib.sourceforge.net/) and \
           Python TkInter https://wiki.python.org/moin/TkInter to run this script")
    raise SystemExit(1)

from argparse import ArgumentParser

class draw_iq:
    def __init__(self, filename, options):
        self.hfile = open(filename, "r")
        self.block_length = options.block
        self.start = options.start
        self.sample_rate = options.sample_rate

        self.datatype = numpy.complex64
        self.sizeof_data = self.datatype().nbytes    # number of bytes per sample in file

        self.axis_font_size = 16
        self.label_font_size = 18
        self.title_font_size = 20
        self.text_size = 22

        # Setup PLOT
        self.fig = figure(1, figsize=(16, 9), facecolor='w')
        rcParams['xtick.labelsize'] = self.axis_font_size
        rcParams['ytick.labelsize'] = self.axis_font_size

        self.text_file     = figtext(0.10, 0.94, ("File: %s" % filename), weight="heavy", size=self.text_size)
        self.text_file_pos = figtext(0.10, 0.88, "File Position: ", weight="heavy", size=self.text_size)
        self.text_block    = figtext(0.40, 0.88, ("Block Size: %d" % self.block_length),
                                     weight="heavy", size=self.text_size)
        self.text_sr       = figtext(0.60, 0.88, ("Sample Rate: %.2f" % self.sample_rate),
                                     weight="heavy", size=self.text_size)
        self.make_plots()

        self.button_left_axes = self.fig.add_axes([0.45, 0.01, 0.05, 0.05], frameon=True)
        self.button_left = Button(self.button_left_axes, "<")
        self.button_left_callback = self.button_left.on_clicked(self.button_left_click)

        self.button_right_axes = self.fig.add_axes([0.50, 0.01, 0.05, 0.05], frameon=True)
        self.button_right = Button(self.button_right_axes, ">")
        self.button_right_callback = self.button_right.on_clicked(self.button_right_click)

        self.xlim = self.sp_iq.get_xlim()

        self.manager = get_current_fig_manager()
        connect('key_press_event', self.click)
        show()

    def get_data(self):
        self.text_file_pos.set_text("File Position: %d" % (self.hfile.tell()//self.sizeof_data))
        try:
            self.iq = numpy.fromfile(self.hfile, dtype=self.datatype, count=self.block_length)
        except MemoryError:
            print("End of File")
        else:
            self.reals = numpy.array([r.real for r in self.iq])
            self.imags = numpy.array([i.imag for i in self.iq])
            self.time = numpy.array([i*(1/self.sample_rate) for i in range(len(self.reals))])

    def make_plots(self):
        # if specified on the command-line, set file pointer
        self.hfile.seek(self.sizeof_data*self.start, 1)

        self.get_data()

        # Subplot for real and imaginary parts of signal
        self.sp_iq = self.fig.add_subplot(2,1,1, position=[0.075, 0.14, 0.85, 0.67])
        self.sp_iq.set_title(("I&Q"), fontsize=self.title_font_size, fontweight="bold")
        self.sp_iq.set_xlabel("Time (s)", fontsize=self.label_font_size, fontweight="bold")
        self.sp_iq.set_ylabel("Amplitude (V)", fontsize=self.label_font_size, fontweight="bold")
        self.plot_iq = plot(self.time, self.reals, 'bo-', self.time, self.imags, 'ro-')
        self.sp_iq.set_ylim([1.5*min([self.reals.min(), self.imags.min()]),
                             1.5*max([self.reals.max(), self.imags.max()])])
        self.sp_iq.set_xlim(self.time.min(), self.time.max())
        draw()

    def update_plots(self):
        self.plot_iq[0].set_data([self.time, self.reals])
        self.plot_iq[1].set_data([self.time, self.imags])
        self.sp_iq.set_ylim([1.5*min([self.reals.min(), self.imags.min()]),
                             1.5*max([self.reals.max(), self.imags.max()])])
        self.sp_iq.set_xlim(self.time.min(), self.time.max())
        draw()

    def click(self, event):
        forward_valid_keys = [" ", "down", "right"]
        backward_valid_keys = ["up", "left"]

        if(find(event.key, forward_valid_keys)):
            self.step_forward()

        elif(find(event.key, backward_valid_keys)):
            self.step_backward()

    def button_left_click(self, event):
        self.step_backward()

    def button_right_click(self, event):
        self.step_forward()

    def step_forward(self):
        self.get_data()
        self.update_plots()

    def step_backward(self):
        # Step back in file position
        if(self.hfile.tell() >= 2*self.sizeof_data*self.block_length ):
            self.hfile.seek(-2*self.sizeof_data*self.block_length, 1)
        else:
            self.hfile.seek(-self.hfile.tell(),1)
        self.get_data()
        self.update_plots()


def find(item_in, list_search):
    try:
        return list_search.index(item_in) != None
    except ValueError:
        return False

def main():
    description = "Takes a GNU Radio complex binary file and displays the I&Q data versus time. You can set the block size to specify how many points to read in at a time and the start position in the file. By default, the system assumes a sample rate of 1, so in time, each sample is plotted versus the sample number. To set a true time axis, set the sample rate (-R or --sample-rate) to the sample rate used when capturing the samples."

    parser = ArgumentParser(conflict_handler="resolve", description=description)
    parser.add_argument("-B", "--block", type=int, default=1000,
            help="Specify the block size [default=%(default)r]")
    parser.add_argument("-s", "--start", type=int, default=0,
            help="Specify where to start in the file [default=%(default)r]")
    parser.add_argument("-R", "--sample-rate", type=float, default=1.0,
            help="Set the sampler rate of the data [default=%(default)r]")
    parser.add_argument("file", metavar="FILE",
            help="Input file with complex samples")

    args = parser.parse_args()

    dc = draw_iq(args.file, args)

if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        pass
